--***********************************************************************
--				SX_FPBAS1.vhd
--	realisation d'un filtre passe bas du 1ere ordre
--						    	
--***********************************************************************

library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

entity SX_FPBAS1 is port (
	RESET      : in  std_logic;	-- reset global
	CLOCK      : in  STD_LOGIC;	-- horloge de la carte (40 MHz)
--	ADDA_INT   : in  STD_LOGIC;	-- signal de fin de conversion
	ADDA_CS    : out STD_LOGIC;	-- selection du convertisseur
	ADDA_ST    : out STD_LOGIC;	-- ordre debut d'echantillonnage
	COM_RD     : out STD_LOGIC;	-- ordre de lecture
	COM_WR     : out STD_LOGIC;	-- ordre d'ecriture
	B_DATA     : inout STD_LOGIC_VECTOR (7 downto 0)  -- data_bus
	);
end SX_FPBAS1;

architecture SX_FPBAS1_arch of SX_FPBAS1 is

-- declaration des differentes entites utilisees

	-- sequenceur gerant les acces au convertisseur
	component Seq_AdDa port(
		reset      : in  std_logic;	-- reset global
		clock      : in  std_logic;	-- 50 ns
		adda_st    : out std_logic;	-- ordre d'echantillonnage
		adda_cs    : out std_logic;	-- selection du convertisseur
		read_o     : out std_logic;	-- ordre de lecture
		enable_o   : out std_logic;	-- activer les donnees en sortie
		write_o    : out std_logic;	-- ordre d'ecrit. vers le conv.
		data_in    : in  std_logic_vector(7 downto 0);	-- bus donnee
		x_data     : out std_logic_vector(7 downto 0)	-- donnee lue
		);
	end component;

-- declaration des signaux intermediaires
	signal dbus_in 			 : std_logic_vector(7 downto 0); -- bus en entree
	signal x_data, x1_data		 : std_logic_vector(7 downto 0); -- echantillon en entree
	signal y_data  			 : std_logic_vector(7 downto 0); -- echantillon en sortie
	signal read_n, writ_n, ecrit   : std_logic;
	
-- definition du coefficient du filtre
constant par : std_logic_vector(7 downto 0) := "01000000";-- 0*2^0+1*2^-1=0.5

begin
	-- interconnexions des 2 entites
DO_CONV: 
	Seq_AdDa port map(RESET,  CLOCK, ADDA_ST, ADDA_CS,
			  read_n, ecrit, writ_n,  dbus_in, x_data);					  
					
	-- calcul de l' echantillon de sortie
	-- realisation d'un filtre passe bas du 1ere ordre
	-- y(data) = a0*x(data) + a1*x(data-1)

	process(reset,ecrit)
	variable coef0,coef1,buf : std_logic_vector(15 downto 0);
	begin
		if reset='0' then 
		      y_data <="00000000";
			 x1_data <="00000000";

		elsif ecrit'event and  ecrit='1' then
		     coef0 := conv_std_logic_vector((conv_signed(conv_integer(par),8) * conv_signed(conv_integer(x_data),8)),16);
			coef1 := conv_std_logic_vector((conv_signed(conv_integer(par),8) * conv_signed(conv_integer(x1_data),8)),16);		
			buf	:= coef0 + coef1;
			y_data <= buf(14 downto 7);
			x1_data <= x_data;
		end if;
	end process;					
					  
	-- Fin Traitement : filtre passe bas 1ere ordre.
						
-- lecture du bus de donnees
	process(B_DATA)
	begin					
		dbus_in <= B_DATA;
	end process;
	
-- positionnement du bus en lecture ou en ecriture suivant "ecr"
ECRI_DAT:
	B_DATA <= y_data when ecrit = '0' else "ZZZZZZZZ"; 
	COM_RD <= read_n;
	COM_WR <= writ_n;

end SX_FPBAS1_arch;