--*****************************************************************
--					Seq_AdDa.vhd                                           
--	Sequenceur destin a la gestion du convertisseur  AD/DA	   				                                              
--*****************************************************************

library IEEE;

use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;

entity Seq_AdDa is port(
	reset      : in  std_logic;	-- reset global
	clock      : in  std_logic;	-- Horloge 40MHz
	adda_st    : out std_logic;	-- ordre d'echantillonnage
	adda_cs    : out std_logic;	-- selection du convertisseur
	read_o     : out std_logic;	-- ordre de lecture
	enable_o   : out std_logic;	-- activer les donnees en sortie
	write_o    : out std_logic;	-- ordre d'ecrit. vers le conv.
	data_in    : in  std_logic_vector(7 downto 0);	-- bus donnee
	x_data     : out std_logic_vector(7 downto 0)	-- donnee lue
	);
end Seq_AdDa;

architecture desc_seqAdDa of Seq_AdDa is

	-- definition des etats du sequenceur
	type   etat is (debut,   start,    attente, lecture,
					fin_lec, deb_ecri, ecrit,  fin_ecri);
	signal etat_seq: etat;	-- etat du sequenceur

	-- declaration des signaux intermediaires
	signal clk_seq : std_logic;
	signal compteur: std_logic_vector(7 downto 0);
	signal indice  : integer range 0 to 63;	-- variable de temporisation

begin

	-- ****************** diviseur d'horloge **********************
DIV_CLK: 
	process (reset, clock) begin
		if reset = '0' then
			compteur <= "00000000";
		elsif clock'event and clock = '1' then
			Compteur <= Compteur + "00000001";
		end if;
	end process;

clk_seq <= Compteur(4);		-- 0.8 s		-- 1.25 MHz La frequence d'horlogedu sequenceur

-- tempo qui determine la frequence d'echantillonage
-- cycle = 80 Ts = 64 micro (le frequence d'echantillonnage 15.625KHz)	
SEQ_IO: 
	process(reset, clk_seq) begin
		if reset = '0' then 
			adda_cs <= '1';		indice   <= 0;
			adda_st <= '1';		enable_o <= '1';
			write_o <= '1';		etat_seq <= debut;
			read_o  <= '1';		x_data   <= "00000000";

		elsif clk_seq = '1' and clk_seq'event then
			case etat_seq is

			when debut =>					-- etat d'attente entre 2 echantillonages
				adda_cs  <= '1';	read_o  <= '1';
				adda_st  <= '1';	write_o <= '1';
				enable_o <= '1';				
				if indice = 6 then 
					etat_seq <= start;
				else 
					etat_seq <= debut;		indice <= indice + 1;
				end if;
				
			when start =>					-- debut conversion (front descendant de start)
				adda_cs  <= '1';		read_o  <= '1';
				adda_st  <= '0';		write_o <= '1';
				enable_o <= '1';
				etat_seq <= attente;	indice  <= 0;

			when attente => 				--  attendre la fin de la conversion
				adda_cs  <= '1';	read_o  <= '1';
				adda_st  <= '0';	write_o <= '1';
				enable_o <= '1';
				if indice = 58 then		 	-- on attend 3 micro-secondes
					etat_seq <= lecture;	indice <= 0;
				else
					etat_seq <= attente;	indice <= indice + 1;
				end if;

			when lecture => 
				adda_cs  <= '0';	read_o  <= '0';
				adda_st  <= '0';	write_o <= '1';
				enable_o <= '1';
				if indice = 3 then 
					etat_seq <= fin_lec;
				else 					-- un cycle lecture = 200 ns
					etat_seq <= lecture;	indice <= indice + 1;
				end if;

			when fin_lec => 
				adda_cs  <= '0';	read_o  <= '0';
				adda_st  <= '0';	write_o <= '1';
				enable_o <= '1';	indice  <= 0;
				x_data   <= data_in;		-- lire l'echantillon			
				etat_seq <= deb_ecri;

			when deb_ecri =>  				-- laisser 200 ns entre ecriture et lecture
				adda_cs  <= '1';	read_o  <= '1';
				adda_st  <= '1';	write_o <= '1';
				enable_o <= '1';
				if indice = 3 then
					etat_seq <= ecrit;		indice <= 0;
				else
					etat_seq <= deb_ecri;	indice <= indice + 1;
				end if;

			when ecrit =>					-- ecriture de l'echnatillon (DAC)
				adda_cs  <= '0';	read_o  <= '1';
				adda_st  <= '1';	write_o <= '0';
				enable_o <= '0'; 			-- valider le bus en sortie
				if indice = 3 then 
					etat_seq <= fin_ecri;
				else 					-- un cycle ecriture = 200 ns
					etat_seq <= ecrit;		indice <= indice + 1;
				end if;

			when fin_ecri => 
				adda_cs  <= '1';	read_o  <= '1';
				adda_st  <= '1';	write_o <= '1';
				enable_o <= '0'; 			-- laisser le bus en sortie 10 ns
				etat_seq <= debut;	indice <= 0;

			when others => 
				null;
			end case;
		end if;
	end process;

end desc_seqAdDa;